#!/bin/bash -e
# Filename: shared-baseimage.sh
# Author:   Matt Bearup
# Details:  Contains functions and resources used by secure base image scripts

export OSVER=$( awk '/Ubuntu/ { print $2 }' /etc/issue )
export OSMAJORVER=${OSVER%%.*}

# Declare arrays holding values for each cloud
declare -a CLOUDS=("public" "china" "usgov" "germany")
declare -a DOMAINS=("cloudapp.net" "chinacloudapp.cn" "usgovcloudapp.net" "azurecloudapp.de")
declare -a STORAGE_DOMAINS=("blob.core.windows.net" "blob.core.chinacloudapi.cn" "blob.core.usgovcloudapi.net" "blob.core.cloudapi.de")

# Create a mapping of domain names to clouds
declare -A DOMAIN_MAPPINGS
for i in ${!CLOUDS[@]}; do
    DOMAIN_MAPPINGS[${DOMAINS[$i]}]=${CLOUDS[$i]}
done

# Create a mapping of storage domain names to clouds
declare -A STORAGE_DOMAIN_MAPPINGS
for i in ${!CLOUDS[@]}; do
    STORAGE_DOMAIN_MAPPINGS[${CLOUDS[$i]}]=${STORAGE_DOMAINS[$i]}
done

# Other variables
export dhclientConf=/etc/dhcp/dhclient.conf
export TRUE=0
export FALSE=1

TryAptInstall() {
    for i in $(seq 3)
    do
        echo "Attempt $i to install $@"
        apt-get update > /dev/null || echo "Error running apt-get update. Attempting to install [$@] anyway"
        if apt-get install -y $@; then
            return
        else
            echo "Failed attempt [$i] to install $@. Sleep and try again..."
            sleep 30
        fi
    done
    echo "Failed to install $@. Program will now exit."
    exit 1
}

function GetReleaseName() {
    lsb_release -a 2> /dev/null | egrep "^Codename" | awk '{print $2}'
}

function IsValidCloudName {
    _cloud="$1"
    if [ -z "$_cloud" ]; then
        return $FALSE
    fi
    for cloud in ${CLOUDS[@]}; do
        if [ "$_cloud" == "$cloud" ]; then
            return $TRUE
        fi
    done
    return $FALSE
}

function GetCloudName() {
    # Returns name of this cloud, or nothing if cloud cannot be identified
    domainName=$1
    if [ -z "$domainName" ]; then
        domainName=$(hostname -f | awk -F. '{printf("%s.%s", $(NF-1), $NF)}') || return
    fi
    echo ${DOMAIN_MAPPINGS[$domainName]}
    return
}

function bail {
    echo "ERROR: $@"
    exit 1
}

function bailIfFileMissing {
	file="$@"
	if [ -z "$file" ]; then
		bail "Unable to check if file is missing because no filename was specified [$file]"
	fi
	if [ ! -e "$file" ]; then
		bail "File [$file] does not exist. Exiting."
	fi
}

function bailIfFileExists {
	file="$@"
	if [ -z "$file" ]; then
		bail "Unable to check if file exists because no filename was specified [$file]"
	fi
	if [ -e "$file" ]; then
		bail "File [$file] was found when it should not exist. Exiting."
	fi
}

# Test for dhclient regression
function IsDhcpOptionBroken {
    sendHostname="send host-name"
    if [ $OSMAJORVER -eq 12 ]; then
        sendHostnameExpected="$sendHostname \"<hostname>\""
    else
        sendHostnameExpected="$sendHostname = gethostname();"
    fi
    if ! grep -q "^\s*$sendHostnameExpected" $dhclientConf; then
        return $TRUE
    else
        return $FALSE
    fi
}

# Fix dhclient regression
function TryFixDhcpOptionIfNeeded {
    if ! IsDhcpOptionBroken; then
        return $TRUE
    fi
    # Config is broken, try to fix it
    if [ $OSMAJORVER -eq 12 ]; then
        echo "Fixing 'send host-name' entry in $dhclientConf"
        sed -i 's|^send host-name = gethostname();|send host-name "<hostname>";|g' $dhclientConf
        # Need to bounce interface to get DNS registration
        ifdown eth0 && ifup eth0
        return $TRUE
    else
        return $FALSE
    fi
}
